"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const zlib_1 = require("zlib");
const error_1 = require("@pnpm/error");
const config_1 = require("@pnpm/config");
const cli_utils_1 = require("@pnpm/cli-utils");
const exportable_manifest_1 = require("@pnpm/exportable-manifest");
const fs_packlist_1 = require("@pnpm/fs.packlist");
const package_bins_1 = require("@pnpm/package-bins");
const fast_glob_1 = __importDefault(require("fast-glob"));
const pick_1 = __importDefault(require("ramda/src/pick"));
const realpath_missing_1 = __importDefault(require("realpath-missing"));
const render_help_1 = __importDefault(require("render-help"));
const tar_stream_1 = __importDefault(require("tar-stream"));
const publish_1 = require("./publish");
const LICENSE_GLOB = 'LICEN{S,C}E{,.*}'; // cspell:disable-line
const findLicenses = fast_glob_1.default.bind(fast_glob_1.default, [LICENSE_GLOB]);
function rcOptionsTypes() {
    return {
        ...cliOptionsTypes(),
        ...(0, pick_1.default)([
            'npm-path',
        ], config_1.types),
    };
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return {
        'pack-destination': String,
        ...(0, pick_1.default)([
            'pack-gzip-level',
        ], config_1.types),
    };
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['pack'];
function help() {
    return (0, render_help_1.default)({
        description: 'Create a tarball from a package',
        usages: ['pnpm pack'],
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Directory in which `pnpm pack` will save tarballs. The default is the current working directory.',
                        name: '--pack-destination <dir>',
                    },
                ],
            },
        ],
    });
}
exports.help = help;
async function handler(opts) {
    const { manifest: entryManifest, fileName: manifestFileName } = await (0, cli_utils_1.readProjectManifest)(opts.dir, opts);
    preventBundledDependenciesWithoutHoistedNodeLinker(opts.nodeLinker, entryManifest);
    const _runScriptsIfPresent = publish_1.runScriptsIfPresent.bind(null, {
        depPath: opts.dir,
        extraBinPaths: opts.extraBinPaths,
        extraEnv: opts.extraEnv,
        pkgRoot: opts.dir,
        rawConfig: opts.rawConfig,
        rootModulesDir: await (0, realpath_missing_1.default)(path_1.default.join(opts.dir, 'node_modules')),
        stdio: 'inherit',
        unsafePerm: true, // when running scripts explicitly, assume that they're trusted.
    });
    if (!opts.ignoreScripts) {
        await _runScriptsIfPresent([
            'prepack',
            'prepare',
        ], entryManifest);
    }
    const dir = entryManifest.publishConfig?.directory
        ? path_1.default.join(opts.dir, entryManifest.publishConfig.directory)
        : opts.dir;
    // always read the latest manifest, as "prepack" or "prepare" script may modify package manifest.
    const { manifest } = await (0, cli_utils_1.readProjectManifest)(dir, opts);
    preventBundledDependenciesWithoutHoistedNodeLinker(opts.nodeLinker, manifest);
    if (!manifest.name) {
        throw new error_1.PnpmError('PACKAGE_NAME_NOT_FOUND', `Package name is not defined in the ${manifestFileName}.`);
    }
    if (!manifest.version) {
        throw new error_1.PnpmError('PACKAGE_VERSION_NOT_FOUND', `Package version is not defined in the ${manifestFileName}.`);
    }
    const tarballName = `${manifest.name.replace('@', '').replace('/', '-')}-${manifest.version}.tgz`;
    const publishManifest = await createPublishManifest({
        projectDir: dir,
        modulesDir: path_1.default.join(opts.dir, 'node_modules'),
        manifest,
        embedReadme: opts.embedReadme,
        catalogs: opts.catalogs ?? {},
    });
    const files = await (0, fs_packlist_1.packlist)(dir, {
        packageJsonCache: {
            [path_1.default.join(dir, 'package.json')]: publishManifest,
        },
    });
    const filesMap = Object.fromEntries(files.map((file) => [`package/${file}`, path_1.default.join(dir, file)]));
    // cspell:disable-next-line
    if (opts.workspaceDir != null && dir !== opts.workspaceDir && !files.some((file) => /LICEN[CS]E(\..+)?/i.test(file))) {
        const licenses = await findLicenses({ cwd: opts.workspaceDir });
        for (const license of licenses) {
            filesMap[`package/${license}`] = path_1.default.join(opts.workspaceDir, license);
        }
    }
    const destDir = opts.packDestination
        ? (path_1.default.isAbsolute(opts.packDestination) ? opts.packDestination : path_1.default.join(dir, opts.packDestination ?? '.'))
        : dir;
    await fs_1.default.promises.mkdir(destDir, { recursive: true });
    await packPkg({
        destFile: path_1.default.join(destDir, tarballName),
        filesMap,
        modulesDir: path_1.default.join(opts.dir, 'node_modules'),
        packGzipLevel: opts.packGzipLevel,
        manifest: publishManifest,
        bins: [
            ...(await (0, package_bins_1.getBinsFromPackageManifest)(publishManifest, dir)).map(({ path }) => path),
            ...(manifest.publishConfig?.executableFiles ?? [])
                .map((executableFile) => path_1.default.join(dir, executableFile)),
        ],
    });
    if (!opts.ignoreScripts) {
        await _runScriptsIfPresent(['postpack'], entryManifest);
    }
    if (opts.dir !== destDir) {
        return path_1.default.join(destDir, tarballName);
    }
    return path_1.default.relative(opts.dir, path_1.default.join(dir, tarballName));
}
exports.handler = handler;
function preventBundledDependenciesWithoutHoistedNodeLinker(nodeLinker, manifest) {
    if (nodeLinker === 'hoisted')
        return;
    for (const key of ['bundledDependencies', 'bundleDependencies']) {
        const bundledDependencies = manifest[key];
        if (bundledDependencies) {
            throw new error_1.PnpmError('BUNDLED_DEPENDENCIES_WITHOUT_HOISTED', `${key} does not work with node-linker=${nodeLinker}`, {
                hint: `Add node-linker=hoisted to .npmrc or delete ${key} from the root package.json to resolve this error`,
            });
        }
    }
}
async function readReadmeFile(projectDir) {
    const files = await fs_1.default.promises.readdir(projectDir);
    const readmePath = files.find(name => /readme\.md$/i.test(name));
    const readmeFile = readmePath ? await fs_1.default.promises.readFile(path_1.default.join(projectDir, readmePath), 'utf8') : undefined;
    return readmeFile;
}
async function packPkg(opts) {
    const { destFile, filesMap, bins, manifest, } = opts;
    const mtime = new Date('1985-10-26T08:15:00.000Z');
    const pack = tar_stream_1.default.pack();
    await Promise.all(Object.entries(filesMap).map(async ([name, source]) => {
        const isExecutable = bins.some((bin) => path_1.default.relative(bin, source) === '');
        const mode = isExecutable ? 0o755 : 0o644;
        if (/^package\/package\.(json|json5|yaml)/.test(name)) {
            pack.entry({ mode, mtime, name: 'package/package.json' }, JSON.stringify(manifest, null, 2));
            return;
        }
        pack.entry({ mode, mtime, name }, fs_1.default.readFileSync(source));
    }));
    const tarball = fs_1.default.createWriteStream(destFile);
    pack.pipe((0, zlib_1.createGzip)({ level: opts.packGzipLevel })).pipe(tarball);
    pack.finalize();
    return new Promise((resolve, reject) => {
        tarball.on('close', () => {
            resolve();
        }).on('error', reject);
    });
}
async function createPublishManifest(opts) {
    const { projectDir, embedReadme, modulesDir, manifest, catalogs } = opts;
    const readmeFile = embedReadme ? await readReadmeFile(projectDir) : undefined;
    return (0, exportable_manifest_1.createExportableManifest)(projectDir, manifest, {
        catalogs,
        readmeFile,
        modulesDir,
    });
}
//# sourceMappingURL=pack.js.map